from overlay import OverlayFS, OverlayFSManager


def mount(directory, lower_dir, upper_dir, mount_table=None):
    """Creates a mount"""
    return OverlayFS.mount(directory, lower_dir, upper_dir,
            mount_table=mount_table)


def list(mount_table=None):
    return OverlayFSManager.list(mount_table=mount_table)


def get(mount_point, mount_table=None):
    return OverlayFSManager.get(mount_point, mount_table=mount_table)



import os
import random

LOWER_ALPHA = "abcdefghijklmnopqrstuvwxyz"
UPPER_ALPHA = LOWER_ALPHA.upper()
ALL_ALPHA = LOWER_ALPHA + UPPER_ALPHA

class PathDoesNotExist(Exception):
    def __init__(self, path):
        message = 'Path "%s" does not exist' % path
        super(PathDoesNotExist, self).__init__(message)

class IsNotADirectory(Exception):
    def __init__(self, path):
        message = 'Path "%s" is not a directory' % path
        super(IsNotADirectory, self).__init__(message)

def ensure_directory(path):
    if os.path.exists(path):
        if os.path.isdir(path):
            return True
        else:
            raise IsNotADirectory(path)
    raise PathDoesNotExist(path)

def ensure_directories(*paths):
    for path in paths:
        ensure_directory(path)

def random_name(length=10):
    letter_list = []
    for i in range(length):
        letter_list.append(random.choice(ALL_ALPHA))
    return ''.join(letter_list)



import re
import subwrap

DIR_REGEX = r'[\w\/\.-]+'
MAIN_REGEX_RAW = r'^(?P<device>%s) on (?P<point>%s) type (?P<fs_type>\w+) \((?P<raw_options>.*)\)$'
MAIN_REGEX = MAIN_REGEX_RAW % (DIR_REGEX, DIR_REGEX)
MAIN_REGEX_OBJ = re.compile(MAIN_REGEX)

def match_entry_line(str_to_match, regex_obj=MAIN_REGEX_OBJ):
    """Does a regex match of the mount entry string"""
    match_obj = regex_obj.match(str_to_match)
    if not match_obj:
        error_message = ('Line "%s" is unrecognized by overlay4u. '
                'This is only meant for use with Ubuntu Linux.')
        raise UnrecognizedMountEntry(error_message % str_to_match)
    return match_obj.groupdict()

def split_mount_options(mount_options_str):
    split_options = mount_options_str.split(',')
    return map(lambda a: a.split('='), split_options)

class UnrecognizedMountEntry(Exception):
    pass

class MountEntry(object):
    @classmethod
    def from_string(cls, entry_str):
        entry_dict = match_entry_line(entry_str)
        options = split_mount_options(entry_dict['raw_options'])
        return cls(entry_dict['point'], entry_dict['device'],
                entry_dict['fs_type'], options)

    def __init__(self, mount_point, device, fs_type, options):
        self.mount_point = mount_point
        self.device = device
        self.fs_type = fs_type
        self.options = options

class MountTable(object):
    @classmethod
    def from_string(cls, list_str, entry_class=None):
        entry_class = entry_class or MountEntry
        # Remove any trailing or preceding newlines
        list_str = list_str.strip()
        entry_list = map(entry_class.from_string, list_str.splitlines())
        return cls(entry_list)

    @classmethod
    def load(cls, entry_class=None):
        response = subwrap.run(['mount'])
        return cls.from_string(response.std_out)

    def __init__(self, entry_list):
        self._entries = entry_list

    def as_list(self, fs_type=None):
        """List mount entries"""
        entries = self._entries
        if fs_type:
            entries = filter(lambda a: a.fs_type == fs_type, entries)
        return entries

    def list_mount_points(self):
        return map(lambda a: a.mount_point, self._entries)

    def find_by_mount_point(self, mount_point):
        found = []
        for entry in self._entries:
            if entry.mount_point == mount_point:
                found.append(entry)
        return found

    def is_mounted(self, mount_point):
        results = self.find_by_mount_point(mount_point)
        return len(results) != 0



from setuptools import setup, find_packages

VERSION = '0.0.3'

LONG_DESCRIPTION = open('README.rst').read()

setup(name='overlay4u',
    version=VERSION,
    description='overlay4u',
    long_description=LONG_DESCRIPTION,
    keywords='overlayfs',
    author='Reuven V. Gonzales',
    author_email='reuven@tobetter.us',
    url='https://github.com/ravenac95/overlay4u',
    license='MIT',
    platforms='Ubuntu',
    packages=find_packages(exclude=['ez_setup', 'examples', 'tests']),
    include_package_data=True,
    zip_safe=False,
    install_requires=[
        'subwrap',
    ],
    entry_points={},
    classifiers=[
        'License :: OSI Approved :: MIT License',
        'Intended Audience :: Developers',
        'Programming Language :: Python',
        'Operating System :: POSIX',
        'Topic :: Software Development :: Build Tools',
    ],
)



import subwrap
from .mountutils import MountTable
from .utils import ensure_directories, random_name


class AlreadyMounted(Exception):
    pass


class InvalidOverlayFS(Exception):
    pass


class FakeMountVerify(object):
    def is_mounted(self, *args):
        return False


class OverlayFS(object):
    @classmethod
    def mount(cls, mount_point, lower_dir, upper_dir, mount_table=None):
        """Execute the mount. This requires root"""
        ensure_directories(mount_point, lower_dir, upper_dir)
        # Load the mount table if it isn't given
        if not mount_table:
            mount_table = MountTable.load()
        # Check if the mount_point is in use
        if mount_table.is_mounted(mount_point):
            # Throw an error if it is
            raise AlreadyMounted()
        # Build mount options
        options = "rw,lowerdir=%s,upperdir=%s" % (lower_dir, upper_dir)
        # Run the actual mount
        subwrap.run(['mount', '-t', 'overlayfs', '-o', options,
            'olyfs%s' % random_name(), mount_point])
        return cls(mount_point, lower_dir, upper_dir)

    @classmethod
    def from_entry(cls, entry):
        options = entry.options
        # FIXME make options an object. This works for now.
        lower_dir = None
        upper_dir = None
        for option in options:
            key = None
            if len(option) == 2:
                key, value = option
            elif len(option) == 1:
                key = option[0]
            if key == 'lowerdir':
                lower_dir = value
            elif key == 'upperdir':
                upper_dir = value
        if not (lower_dir and upper_dir):
            raise InvalidOverlayFS('Upper and lower directories '
                    'do not seem to be defined')
        return cls(entry.mount_point, lower_dir, upper_dir)

    def unmount(self):
        subwrap.run(['umount', self.mount_point])

    def __init__(self, mount_point, lower_dir, upper_dir):
        self.mount_point = mount_point
        self.lower_dir = lower_dir
        self.upper_dir = upper_dir

    def __repr__(self):
        return '<OverlayFS "%s">' % self.mount_point


class OverlayFSDoesNotExist(Exception):
    pass


class OverlayFSManager(object):
    @classmethod
    def list(cls, mount_table=None):
        if not mount_table:
            mount_table = MountTable.load()
        mount_entries = mount_table.as_list(fs_type='overlayfs')
        overlay_entries = []
        for entry in mount_entries:
            overlay_entries.append(OverlayFS.from_entry(entry))
        return overlay_entries

    @classmethod
    def get(cls, mount_point, mount_table=None):
        overlayfs_list = cls.list(mount_table=mount_table)
        for overlayfs in overlayfs_list:
            if overlayfs.mount_point == mount_point:
                return overlayfs
        raise OverlayFSDoesNotExist('Overlay with mount point, '
                '"%s", does not exist' % mount_point)
